/*SpringEmbedders - A program to visualize graphs
in non-euclidean geometries using a Springembedder Algorithm

Copyright (C) 2005  Markus Lipp

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

/*

This file is based on:

/********************************************
* Object Orientated NeHeGL Using Base Class *
* Author: Andreas Oberdorfer           2004 *
********************************************

It was modified by Markus Lipp, 10.06.2005

Copyright Notice by Nehe: "The lessons on this page may contain mistakes, poor commenting, and should not be considered the best resource to learn OpenGL from. What you do with the code is up to you. I am merely trying to make the learning process a little easier for those people new to OpenGL."

*/




#include "GLWindow.h"													// Header File For The GLWindow Basecode

#include <gl/gl.h>														// Header File For The OpenGL32 Library
#include <gl/glu.h>														// Header File For The GLu32 Library


// Initialize Class Default Values
GL_Window::GL_Window()													// Class Constructor
{
	m_WindowPosX	= 0;												// Window X Position
	m_WindowPosY	= 0;												// Window Y Position
	m_WindowWidth	= 1024;												// Window Width
	m_WindowHeight	= 768;												// Window Height
	m_ScreenWidth	= 1024;												// Screen Width
	m_ScreenHeight	= 768;												// Screen Height
	m_BitsPerPixel	= 32;												// Bits Per Pixel
	m_IsFullScreen	= false;											// Fullscreen

	m_hWnd = 0;
	m_hDC = 0;
	m_hRC = 0;

	m_zNear=10.0f;
	m_zFar=400.0f;
	m_IsCursorConfined=false;
}

void GL_Window::ConfineCursor()
{
	if (m_IsCursorConfined==false)
	{
		RECT rcClip;           // new area for ClipCursor
		rcOldClip;        // previous area for ClipCursor
	 
		GetClipCursor(&rcOldClip); 
	 
		// Get the dimensions of the application's window. 
		 
		GetWindowRect(m_hWnd, &rcClip); 
		 
		// Confine the cursor to the application's window. 
		 
		ClipCursor(&rcClip); 
		m_IsCursorConfined=true;
	}
 
	
}

void GL_Window::ReleaseCursor()
{
	ClipCursor(&rcOldClip); 
	m_IsCursorConfined=false;
}



// Get Window Dimensions Depending On Screen Mode (FullScreen / Windowed)
int GL_Window::GetWidth()
{
	if (m_IsFullScreen == true)
	{
		return m_ScreenWidth;
	}
	else
	{
		return m_WindowWidth;
	}
}
int GL_Window::GetHeight()
{
	if (m_IsFullScreen == true)
	{
		return m_ScreenHeight;
	}
	else
	{
		return m_WindowHeight;
	}
}

// Set Window Dimensions Depending On Screen Mode (FullScreen / Windowed)
void GL_Window::SetWidth(int width)
{
	if (m_IsFullScreen == true)
	{
		m_ScreenWidth = width;
	}
	else
	{
		m_WindowWidth = width;
	}
}
void GL_Window::SetHeight(int height)
{
	if (m_IsFullScreen == true)
	{
		m_ScreenHeight = height;
	}
	else
	{
		m_WindowHeight = height;
	}
}

// Get Window Position Depending On Screen Mode (FullScreen / Windowed)
int GL_Window::GetPosX()
{
	if (m_IsFullScreen == false)
	{
		return m_WindowPosX;
	}
	return 0;
}
int GL_Window::GetPosY()
{
	if (m_IsFullScreen == false)
	{
		return m_WindowPosY;
	}
	return 0;
}

// Set Window Position Depending On Screen Mode (FullScreen / Windowed)
void GL_Window::SetPosX(int x)
{
	if (m_IsFullScreen == false)
	{
		m_WindowPosX = x;
	}
}
void GL_Window::SetPosY(int y)
{
	if (m_IsFullScreen == false)
	{
		m_WindowPosY = y;
	}
}

// Reshape The Window When It's Resized
void GL_Window::ReshapeGL()
{
	GLsizei width = GetWidth();
	GLsizei height = GetHeight();
	glViewport(0, 0, width, height);									// Reset The Current Viewport
	glMatrixMode(GL_PROJECTION);										// Select The Projection Matrix
	glLoadIdentity();													// Reset The Projection Matrix
	gluPerspective(45.0f, (GLfloat)width / (GLfloat)height, m_zNear, m_zFar);	// Calculate The Aspect Ratio Of The Window
	glMatrixMode(GL_MODELVIEW);											// Select The Modelview Matrix
	glLoadIdentity();													// Reset The Modelview Matrix
}

// Change The Screen Resolution
bool GL_Window::ChangeScreenResolution()
{
	DEVMODE dmScreenSettings;											// Device Mode
	ZeroMemory(&dmScreenSettings, sizeof(DEVMODE));						// Make Sure Memory Is Cleared
	dmScreenSettings.dmSize			= sizeof(DEVMODE);					// Size Of The Devmode Structure
	dmScreenSettings.dmPelsWidth	= GetWidth();						// Select Window Width
	dmScreenSettings.dmPelsHeight	= GetHeight();						// Select Window Height
	dmScreenSettings.dmBitsPerPel	= m_BitsPerPixel;					// Select Bits Per Pixel
	dmScreenSettings.dmFields		= DM_BITSPERPEL | DM_PELSWIDTH | DM_PELSHEIGHT;
	if (ChangeDisplaySettings(&dmScreenSettings, CDS_FULLSCREEN) != DISP_CHANGE_SUCCESSFUL)
	{
		return false;													// Display Change Failed, Return false
	}
	return true;														// Display Change Was Successful, Return true
}

// Create OpenGL Window
bool GL_Window::Create(const char * window_title, bool full_screen, const char * class_name, HINSTANCE h_instance, LPVOID lpParam)
{
	m_IsFullScreen = full_screen;

	PIXELFORMATDESCRIPTOR pfd =											// pfd Tells Windows How We Want Things To Be
	{
		sizeof(PIXELFORMATDESCRIPTOR),									// Size Of This Pixel Format Descriptor
		1,																// Version Number
		PFD_DRAW_TO_WINDOW	|											// Format Must Support Window
		PFD_SUPPORT_OPENGL	|											// Format Must Support OpenGL
		PFD_STEREO			|											// Only supported by few Cards (such as ASUS and ELSA GeForce series, and GF Quadro series) with Stereo-glasses
		PFD_DOUBLEBUFFER,												// Must Support Double Buffering
		PFD_TYPE_RGBA,													// Request An RGBA Format
		m_BitsPerPixel,													// Select Our Color Depth
		0, 0, 0, 0, 0, 0,												// Color Bits Ignored
		0,																// No Alpha Buffer
		0,																// Shift Bit Ignored
		0,																// No Accumulation Buffer
		0, 0, 0, 0,														// Accumulation Bits Ignored
		16,																// 16Bit Z-Buffer (Depth Buffer)
		0,																// No Stencil Buffer
		0,																// No Auxiliary Buffer
		PFD_MAIN_PLANE,													// Main Drawing Layer
		0,																// Reserved
		0, 0, 0															// Layer Masks Ignored
	};

	DWORD windowStyle = WS_OVERLAPPEDWINDOW;							// Define Our Window Style
	DWORD windowExtendedStyle = WS_EX_APPWINDOW;						// Define The Window's Extended Style

	if (m_IsFullScreen == true)											// Fullscreen Requested, Try Changing Video Modes
	{
		if (ChangeScreenResolution() == false)
		{																// Fullscreen Mode Failed. Run In Windowed Mode Instead
			MessageBox(HWND_DESKTOP, "Mode Switch Failed.\nRunning In Windowed Mode.", "Error", MB_OK | MB_ICONEXCLAMATION);
			m_IsFullScreen = false;										// Set isFullscreen To false (Windowed Mode)
		}
		else															// Otherwise (If Fullscreen Mode Was Successful)
		{
			ShowCursor(false);											// Turn Off The Cursor
			windowStyle = WS_POPUP;										// Set The WindowStyle To WS_POPUP (Popup Window)
			windowExtendedStyle |= WS_EX_TOPMOST;						// Set The Extended Window Style To WS_EX_TOPMOST
		}																// (Top Window Covering Everything Else)
	}

	// Define Our Window Coordinates
	RECT windowRect = {GetPosX(), GetPosY(), GetPosX() + GetWidth(), GetPosY() + GetHeight()};
	if (m_IsFullScreen == false)										// If Fullscreen Was Not Selected
	{	// Adjust Window, Account For Window Borders
		AdjustWindowRectEx(&windowRect, windowStyle, 0, windowExtendedStyle);
		// Check If Window's Left-Top Corner Is Inside The DeskTop
		if (windowRect.left < 0)										// If Window X Position Negative
		{
			windowRect.right -= windowRect.left;						// Correct Right Position
			windowRect.left = 0;										// Move Window To X Position 0
		}
		if (windowRect.top < 0)											// If Window Y Position Negative
		{
			windowRect.bottom -= windowRect.top;						// Correct Bottom Position
			windowRect.top = 0;											// Move Window To Y Position 0
		}
	}

	// Create The OpenGL Window
	m_hWnd = CreateWindowEx(windowExtendedStyle,						// Extended Style
							class_name,									// Class Name
							window_title,								// Window Title
							windowStyle,								// Window Style
							windowRect.left, windowRect.top,			// Window X,Y Position
							windowRect.right - windowRect.left,			// Window Width
							windowRect.bottom - windowRect.top,			// Window Height
							HWND_DESKTOP,								// Desktop Is Window's Parent
							0,											// No Menu
							h_instance,									// Pass The Window Instance
							lpParam);									// Pass The Application Class Context

	while (m_hWnd != 0)													// Was Window Creation A Success?
	{
		m_hDC = GetDC(m_hWnd);											// Grab A Device Context For This Window
		if (m_hDC == 0)													// Did We Get A Device Context?
		{																// Failed
			break;														// GoTo End Of while (m_hWnd != 0) Block
		}
		GLuint PixelFormat = ChoosePixelFormat(m_hDC, &pfd);			// Find A Compatible Pixel Format
		if (PixelFormat == 0)											// Did We Find A Compatible Format?
		{																// Failed
			break;														// GoTo End Of while (m_hWnd != 0) Block
		}
		if (SetPixelFormat(m_hDC, PixelFormat, &pfd) == false)			// Try To Set The Pixel Format
		{																// Failed
			break;														// GoTo End Of while (m_hWnd != 0) Block
		}
		m_hRC = wglCreateContext(m_hDC);								// Try To Get A Rendering Context
		if (m_hRC == 0)													// Did We Get A Rendering Context?
		{																// Failed
			break;														// GoTo End Of while (m_hWnd != 0) Block
		}
		if (wglMakeCurrent(m_hDC, m_hRC) == false)						// Make The Rendering Context Our Current Rendering Context
		{																// Failed
			break;														// GoTo End Of while (m_hWnd != 0) Block
		}

		ShowWindow(m_hWnd, SW_NORMAL);									// Make The Window Visible
		ReshapeGL();													// Reshape Our GL Window
		return true;													// Window Creating Was A Success
	}																	// Initialization Will Be Done In WM_CREATE

	Destroy();															// Free Resources
	return false;														// If Any Error Return false
}

// Destroy OpenGL Window
void GL_Window::Destroy()									// Destroy The OpenGL Window & Release Resources
{
	if (m_hWnd != 0)										// Does The Window Have A Handle?
	{
		if (m_hDC != 0)										// Does The Window Have A Device Context?
		{
			wglMakeCurrent(m_hDC, 0);						// Set The Current Active Rendering Context To Zero
			if (m_hRC != 0)									// Does The Window Have A Rendering Context?
			{
				wglDeleteContext(m_hRC);					// Release The Rendering Context
				m_hRC = 0;									// Zero The Rendering Context
			}
			ReleaseDC(m_hWnd, m_hDC);						// Release The Device Context
			m_hDC = 0;										// Zero The Device Context
		}
		DestroyWindow(m_hWnd);								// Destroy The Window
		m_hWnd = 0;											// Zero The Window Handle
	}

	if (m_IsFullScreen)										// Is Window In Fullscreen Mode
	{
		ChangeDisplaySettings(NULL, 0);						// Switch Back To Desktop Resolution
		ShowCursor(true);									// Show The Cursor
	}
}
