/*SpringEmbedders - A program to visualize graphs
in non-euclidean geometries using a Springembedder Algorithm

Copyright (C) 2005  Markus Lipp

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

/*

This file is based on:

/********************************************
* Object Orientated NeHeGL Using Base Class *
* Author: Andreas Oberdorfer           2004 *
********************************************

It was modified by Markus Lipp, 10.06.2005

Copyright Notice by Nehe: "The lessons on this page may contain mistakes, poor commenting, and should not be considered the best resource to learn OpenGL from. What you do with the code is up to you. I am merely trying to make the learning process a little easier for those people new to OpenGL."

*/



#ifndef	__GL_WINDOW_H__
#define	__GL_WINDOW_H__



#include <windows.h>													// Header File For Windows


/** Window Creation Info*/
class GL_Window															
{
public:

	/** Confine mouse cursor*/
	void ConfineCursor();
	/**release mouse cursor*/
	void ReleaseCursor();

	/** Class Constructor*/
	GL_Window();

	/** Create OpenGL Window*/
	bool	Create(const char * window_title, bool full_screen, const char * class_name, HINSTANCE h_instance, LPVOID lpParam);
	/** Destroy OpenGL Window*/
	void	Destroy();
	/** Change The Screen Resolution*/
	bool	ChangeScreenResolution();
	/** Reshape The Window When It's Resized*/
	void	ReshapeGL();
	/** Swap Buffers (Double Buffering)*/
	void	SwapBuffers() { ::SwapBuffers(m_hDC); }
	/** Set Window Position Depending On Screen Mode (FullScreen / Windowed)*/
	void	SetPosX(int x);
	/** Sign Extend If Used With WORD Parameter*/
	void	SetPosX(unsigned short x) { SetPosX((signed short)x); }		
	void	SetPosY(int y);
	/** Sign Extend If Used With WORD Parameter*/
	void	SetPosY(unsigned short y) { SetPosY((signed short)y); }		
	/** Get Window Dimensions Depending On Screen Mode (FullScreen / Windowed)*/
	int		GetWidth();
	/** Get Window Dimensions Depending On Screen Mode (FullScreen / Windowed)*/
	int		GetHeight();
	/** Set Window Dimensions Depending On Screen Mode (FullScreen / Windowed)*/
	void	SetWidth(int width);
	/** Set Window Dimensions Depending On Screen Mode (FullScreen / Windowed)*/
	void	SetHeight(int height);
	/** Get Window Position Depending On Screen Mode (FullScreen / Windowed)*/
	int		GetPosX();
	/** Get Window Position Depending On Screen Mode (FullScreen / Windowed)*/
	int		GetPosY();
	/** Set BitsPerPixel To 16*/
	void	SetHiColor()	{ m_BitsPerPixel = 16; }
	/** Set BitsPerPixel To 32*/
	void	SetTrueColor()	{ m_BitsPerPixel = 32; }
	/** Class Conversion Operator For Window Handle (GL_Window Can Be Used As A Window Handle)*/
	operator HWND() { return m_hWnd; }

	/** return near clipping plane*/
	float getzNear() {return m_zNear;};
	/** return far clipping plane*/
	float getzFar() {return m_zFar;};

	/** get HDC*/
	HDC getHDC() {return m_hDC;};

private:
	/** Window Handle*/
	HWND	m_hWnd;
	/** Device Context*/
	HDC		m_hDC;	
	/** Rendering Context*/
	HGLRC	m_hRC;														

	int		m_WindowPosX;												// X Position Of Window
	int		m_WindowPosY;												// Y Position Of Window
	int		m_WindowWidth;												// Width Of Window
	int		m_WindowHeight;												// Height Of Window
	int		m_ScreenWidth;												// Width Of FullScreen
	int		m_ScreenHeight;												// Height Of FullScreen
	int		m_BitsPerPixel;												// Bits Per Pixel
	bool	m_IsFullScreen;												// FullScreen


	RECT    rcOldClip;
	bool m_IsCursorConfined;
	

	float m_zNear, m_zFar;
};


#endif	// __GL_WINDOW_H__
