/*SpringEmbedders - A program to visualize graphs
in non-euclidean geometries using a Springembedder Algorithm

Copyright (C) 2005  Markus Lipp

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#include ".\gl_springembedders.h"
#include <stdio.h>
#include <io.h>
#include <string>


				
#pragma comment(lib, "opengl32.lib")							// Search For OpenGL32.lib While Linking
#pragma comment(lib, "glu32.lib")								// Search For GLu32.lib While Linking
#pragma comment(lib, "glaux.lib")								// Search For GLaux.lib While Linking

/**Return Microsoft Error String*/
char *GetErrorStringMS(DWORD errorValue)
{
	LPVOID lpMsgBuf;

	FormatMessage( 
		FORMAT_MESSAGE_ALLOCATE_BUFFER | 
		FORMAT_MESSAGE_FROM_SYSTEM|FORMAT_MESSAGE_MAX_WIDTH_MASK,
		NULL,
		errorValue,
		MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
		(LPTSTR) &lpMsgBuf,
		0,
		NULL 
	);

	//String message;
	char *message;
	if (lpMsgBuf)
	{
		message = (char *)lpMsgBuf;

		// HACK: we cant free the buffer here, otherwise there will
		// be access violation errors!
		// Free the buffer.
		//LocalFree( lpMsgBuf );
	}
	else
	{
		message = "No error message returned";
	}

	return message;
}

/** Create The Derived GL_SpringEmbedders Class And Return A Pointer To The GL_Application Base Class*/
GL_Application * GL_Application::Create(const char * class_name)
{
	GL_SpringEmbedders * example = new GL_SpringEmbedders(class_name);
	return reinterpret_cast<GL_Application *>(example);
}



void GL_SpringEmbedders::initXerces()
{
 // initialize the XML library.
  XMLPlatformUtils::Initialize();
  // create new parser instance.
  parser = new XercesDOMParser;
  if (parser)
  {
    parser->setValidationScheme(XercesDOMParser::Val_Always); 

    parser->setDoNamespaces(false);
    parser->setDoSchema(false);

   errHandler = (DOMTreeErrorReporter*) new DOMTreeErrorReporter();
    parser->setErrorHandler(errHandler);

    parser->setCreateEntityReferenceNodes(false);

    //parser->setToCreateXMLDeclTypeNode(true);
  }

}




// Class Constructor
GL_SpringEmbedders::GL_SpringEmbedders(const char * class_name) : GL_Application(class_name)
{
													// Set Starting Angle To Zero
}

bool GL_SpringEmbedders::Initialize()									// Any GL Init Code & User Initialiazation Goes Here
{
	//Set variables to default values
	m_bFirsttimeDisplaymode[0]=true;
	m_bFirsttimeDisplaymode[1]=true;
	m_bFirsttimeDisplaymode[2]=true;
	
	m_bHyperbolicDragMode=false;
	initialized=true;
	m_rotateEnabled=false;
	m_bSelected=false;
	m_dragEnabled=false;
	m_bStrictParsing=true;
	m_iElapsedtime=0;
	iterationscount=0;

	//mouse variables init
	POINT pos;
	GetCursorPos(&pos);	
	m_iDeltaMouseX =pos.x;
	m_iDeltaMouseY = pos.y;

	// OpenGL initialization
	glClearColor(0.0f, 0.0f, 0.0f, 0.5f);						// Black Background
	glClearDepth(1.0f);											// Depth Buffer Setup
	glDepthFunc(GL_LEQUAL);										// The Type Of Depth Testing (Less Or Equal)
	glEnable(GL_DEPTH_TEST);									// Enable Depth Testing
	glShadeModel(GL_SMOOTH);									// Select Smooth Shading
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);			// Set Perspective Calculations To Most Accurate

	glHint(GL_POINT_SMOOTH_HINT, GL_NICEST);
	glHint(GL_LINE_SMOOTH_HINT, GL_NICEST);
	glHint(GL_POLYGON_SMOOTH_HINT, GL_NICEST);

	ResizeDraw(false);											// Disable Redraw During Window Resize

	//set opengl fog-mode
	glFogi(GL_FOG_MODE,GL_LINEAR);
	glFogf(GL_FOG_START,m_Window.getzNear()+30);
	glFogf(GL_FOG_END,m_Window.getzFar()-30);

	GLfloat FogColor[]= { 1,1,1,1 };
	glFogfv(GL_FOG_COLOR,FogColor);


	//reset current rotation matrix
	glPushMatrix();
	glLoadIdentity();
	glGetFloatv(GL_MODELVIEW_MATRIX,m_pfRotationMatrix);
	glPopMatrix();

	//reset current camera offset
	m_vecCameraDelta=CVector3(0,0,0);

	
    //Init main classes, set default to Spherical-geometry
	m_pGraph = new cGraph;	
	m_pGraph->setIs2DRepresentation(false);
	m_pGeometry = (cGeometry*) new cSphericalGeometry;
	m_pSpring = new cSpring;	

	//Load random graph
	m_pGraph->LoadGraph();	
	m_pGraph->RandomizePositions(m_pGeometry);

	//set color-scheme
	m_vecColorMain =			(CVector3(0,72,152)).div(255.0f);
	m_vecColorBackground =	(CVector3(255,255,255)).div(255.0f);
	m_vecColorContrast1 =	(CVector3(255,178,0)).div(255.0f);
	m_vecColorContrast2 =	(CVector3(255,51,0)).div(255.0f);

	m_vecColorFogMain = m_vecColorMain.LinearInterpolate(m_vecColorBackground,0.8);
	m_vecColorFogContrast1 = m_vecColorContrast1.LinearInterpolate(m_vecColorBackground,0.8);
	m_vecColorFogContrast2 = m_vecColorContrast2.LinearInterpolate(m_vecColorBackground,0.8);

	m_vecColorMainLight = (CVector3(0,138,244)).div(255.0f);
	m_vecColorContrast1Light = m_vecColorContrast1.LinearInterpolate(m_vecColorBackground,0.4);
	m_vecColorContrast2Light = m_vecColorContrast2.LinearInterpolate(m_vecColorBackground,0.4);

	//set OpenGL material-properties
	GLfloat material_ambient[] = { 1, 1, 1, 1 };
    GLfloat material_diffuse[] = { 1, 1, 1, 1 };
    GLfloat material_specular[] = { 0, 0, 0, 0 };
    GLfloat material_shininess[] = { 0 };
	glMaterialfv(GL_FRONT, GL_AMBIENT, material_ambient);
    glMaterialfv(GL_FRONT, GL_DIFFUSE, material_diffuse);
    glMaterialfv(GL_FRONT, GL_SPECULAR, material_specular);
    glMaterialfv(GL_FRONT, GL_SHININESS, material_shininess);
	
	//initialize user interface system
	initCEGUI();		
	CEGUI::System::getSingleton().injectMousePosition(pos.x,pos.y);

	//Set floating-point exception behavior
	_clearfp(); // always call _clearfp before 
	//enabling/unmasking a FPU exception
	/*_controlfp( _EM_ZERODIVIDE, _MCW_EM );
	_controlfp( _EM_INVALID, _MCW_EM );
	_controlfp( _EM_DENORMAL, _MCW_EM );
	_controlfp( _EM_OVERFLOW, _MCW_EM );
	_controlfp( _EM_UNDERFLOW, _MCW_EM );
	_controlfp( _EM_INEXACT, _MCW_EM );*/

	//init xml-parser
	initXerces();

	//Init Sphere-Displaylist
	GLUquadricObj* Sphere;
	Sphere = gluNewQuadric(  );
	m_uiSphereDL = glGenLists(1);

	glNewList(m_uiSphereDL,GL_COMPILE);	
		gluSphere(Sphere, 1,12,12);
	glEndList();

	return true;												// Return true (Initialization Successful)
}

void GL_SpringEmbedders::SetAmbientMaterialColor(CVector3 color)		
{
	float col[4];
	col[0]=color.x;col[1]=color.y;col[2]=color.z;col[3]=0;	
	glMaterialfv(GL_FRONT, GL_AMBIENT, col);
}

void GL_SpringEmbedders::SetColor(CVector3 color)
{
	glColor3f(color.x,color.y,color.z);
}

void GL_SpringEmbedders::SetFogColor(CVector3 color)
{
	float col[4];
	col[0]=color.x;col[1]=color.y;col[2]=color.z;col[3]=0;	
	glFogfv(GL_FOG_COLOR,col);
}


void GL_SpringEmbedders::Deinitialize()									// Any User DeInitialization Goes Here
{
	if ( m_GLrenderer )
		delete m_GLrenderer;

	delete(m_pGraph);
	delete(m_pGeometry);
	delete(m_pSpring);

	KillFont();

	 XMLPlatformUtils::Terminate();
}

void GL_SpringEmbedders::Update(DWORD milliseconds)						// Perform Motion Updates
{	
	
	//Calculate Iterations/Second
	m_iElapsedtime+=milliseconds;
	if (m_iElapsedtime>1000)
	{
		m_iElapsedtime=0;
		OutPutStaticText("dp_st13", (float)iterationscount);
		iterationscount=0;
	}

	//should we prevent graph-movement during this update-call?
	if (m_bNoTimeStep)
	{
		milliseconds=0;
		m_bNoTimeStep=false;
	}

	//Handle cursor Positions
	POINT pos;
	GetCursorPos(&pos);
	
	m_iDeltaMouseX = -m_iDeltaMouseX + pos.x;
	m_iDeltaMouseY = -m_iDeltaMouseY + pos.y;

	//calculate mousedelta relative to screen-dimensions
	float m_iDeltaMouseXRel = ((float)m_iDeltaMouseX) / (float)m_Window.GetWidth();
	float m_iDeltaMouseYRel = ((float)m_iDeltaMouseY) / (float)m_Window.GetWidth();

	CEGUI::System::getSingleton().injectMousePosition(pos.x-m_Window.GetPosX(),pos.y-m_Window.GetPosY());

	//Execute Spring Algorithm
	if (!FLOAT_EQ(m_pGraph->getTimeStepMult(),0))
	{		
		m_pGraph->SpringAlgorithm(m_pGeometry, m_pSpring, (float)(milliseconds) / 1000.0f, 1.0f);
		iterationscount++;
	}

	// Calculate current transform matrix, we need it often
	// Reset The Modelview Matrix
	glLoadIdentity ();

	//set up OpenGL camera based on user-interactions
	gluLookAt(-6.5+m_vecCameraDelta.x,0+m_vecCameraDelta.y,50+m_vecCameraDelta.z
		,-6.5+m_vecCameraDelta.x,0+m_vecCameraDelta.y,0+m_vecCameraDelta.z
		,0,1,0);

    //Multiply camera matrix with rotation-matrix	
	glMultMatrixf(m_pfRotationMatrix);

	//get calculated modelview matrix
	GLfloat transform[16];
	glGetFloatv(GL_MODELVIEW_MATRIX,transform);


	//User interaction logic for mouse-clicks
	if (m_lbuttonup)
	{
		m_dragEnabled=false;
		m_bFirstTimeDragging=false;
	}

	if (m_rbuttonup)
	{
        m_rotateEnabled=false;		
	}
	
	if (m_wheel_moved)
	{
		m_vecCameraDelta=m_vecCameraDelta.add(0,0,-(float)m_zDelta/20.0f);    

	}

	if (m_mbuttondown)
	{

			m_vecCameraDelta=m_vecCameraDelta.add(-m_iDeltaMouseX*0.1,m_iDeltaMouseY*0.1,0);            
	}


	//Project cursor position to viewspace
	GLdouble modelMatrix[16];
	GLdouble projMatrix[16];
	GLint viewport[4];
	GLdouble Xd, Yd, Zd,
				X0, Y0, Z0;

	glGetDoublev(GL_MODELVIEW_MATRIX,modelMatrix);
	glGetDoublev(GL_PROJECTION_MATRIX,projMatrix);
	glGetIntegerv(GL_VIEWPORT,viewport);

	//get OpenGL-Window coordinates of mouseposition
	double posx = pos.x-m_Window.GetPosX();
	double posy = m_Window.GetHeight()-pos.y+m_Window.GetPosY();

	//Handle picking

	float A,B,C,D;

	//Get Ray trough mouse cursor
	gluUnProject(posx, posy, 0.5, modelMatrix, projMatrix, viewport,
							&X0, &Y0, &Z0);


	//get z-Value of point (0,0,0)
	gluProject(0, 0, 0, modelMatrix, projMatrix, viewport,
							&Xd, &Yd, &Zd);

	//get mouse-position in viewspace with viewspace-z=0
	gluUnProject(posx, posy, Zd, modelMatrix, projMatrix, viewport,
							&Xd, &Yd, &Zd);

	//startpoint of ray
	CVector3 p0 ( (float)X0, (float)Y0, (float)Z0 );
	//point on ray
	CVector3 pd ((float)Xd,(float)Yd,(float)Zd);
	//Direction of Ray
	CVector3 d = p0.sub(pd); 
	d.normalize();

    //Test for selections
	//Select new objects only if
	// - not in drag mode
	// - not in rotation mode
	// - neither r or l mouse button down
	if ( (!m_dragEnabled) & (!m_rotateEnabled) & (!m_rbuttondown) & (!m_lbuttondown) )
	{


		//initialize values for nearest-node choosing
		float nearest = -99999;	
		m_iNearestnode=0;
		m_bSelected=false;

		//get current modelview matrix
		GLfloat transform[16];
		glGetFloatv(GL_MODELVIEW_MATRIX,transform);

		//step trough each node
		for (unsigned int i=0; i< m_pGraph->getNumNodes(); i++)
		{
            //get node position = center of sphere pc
			CVector3 pc = m_pGeometry->ProjectToEuclid(m_pGraph->getPosition(i));
			//Vector Linepoint0 to Sphere center
			CVector3 p0pc = p0.sub(pc); 

			//calculate radius of sphere
			float r = 0.5+( m_pGraph->getWeight(i)/m_pGraph->getMaxScale())*1.5f ;
			if (m_iDisplayMode == eHyperbolic)
			{
				r *= 1.0-1.0*( __max( (pc).length() / ((cHyperbolicGeometry*)m_pGeometry)->getPoincareRadius(),0));
				r+=0.2;

			}

			//Sphere Ray intersection
			A = d.dot(d);
			B = (d.mult(2)).dot( p0pc);
			C = p0pc.dot(p0pc)-(r*r);
			D= (B*B) - (4*A*C); //Discriminant

			//transform position of current node to viewspace for nearest-node comparisons
			CVector3 PosTransformed = pc;
			PosTransformed.TransformByMatrix(transform);

			//Only select nearest node
			//Discriminant D > 0 => ray hits sphere
			if ( (D>0) & (PosTransformed.z>nearest) & (PosTransformed.z<-m_Window.getzNear()) )
			{
				//set old nearest node to not-selected
				m_pGraph->SetIsSelected(m_iNearestnode,false);
				//set current node to nearest node
				m_pGraph->SetIsSelected(i,true);				
				m_iNearestnode=i;
				//set current-node z to nearest z
				nearest = PosTransformed.z;
				m_bSelected=true;
				m_iSelectedNode=i;
			}
			else
			{
				//no hit, or not nearest hit
				m_pGraph->SetIsSelected(i,false);
			}

		}

	}

	//Dragging mode?
	//only in dragging-mode if
	// - left mouse button is down
	// - a node is selected
	if ( (m_lbuttondown) & (m_bSelected) )
	{
		//check if this is the first time we are in dragging mode
		//needed to initialize states when in sphere-displaymode
		if (m_dragEnabled==false)
			m_bFirstTimeDragging=true;
		m_dragEnabled=true;

		//get current modelview matrix
		GLfloat transform[16];
		glGetFloatv(GL_MODELVIEW_MATRIX,transform);

		//calculate position of mousecursor projected to the current geometry
		if (m_iDisplayMode==eEuclidic3D)
		{
            //get position of selected node		
			CVector3 pc = m_pGeometry->ProjectToEuclid(m_pGraph->getPosition(m_iNearestnode));

			//project to device-coordinates to get device-z value
			gluProject(pc.x, pc.y, pc.z, modelMatrix, projMatrix, viewport,
							&Xd, &Yd, &Zd);

			//unproject mouseposition to viewspace with previously fetched device-z-value
			//i.e. the mouse-z equals the node-z
			gluUnProject(posx, posy, Zd, modelMatrix, projMatrix, viewport,
							&Xd, &Yd, &Zd);

			//Set new node position to unprojected mouseposition
			m_pGraph->SetPosition(m_iNearestnode,CVector3(Xd,Yd,Zd),m_pGeometry);

		}

		if (m_iDisplayMode==eHyperbolic)
		{
			//Get Poincare Position of mouse-position, set selected node position to this position
			m_pGraph->SetPosition(m_iNearestnode,((cHyperbolicGeometry*)m_pGeometry)->ProjectFromEuclidInverseCOA(pd),m_pGeometry);
		}

		if (m_iDisplayMode==eSpherical)
		{
			//intersect mouse-ray with sphere

			//radius = sphere-geometry radius
			float radius = ((cSphericalGeometry*)m_pGeometry)->getRadius();

			//sphere center
			CVector3 pc (0,0,0);
			CVector3 p0pc = pd.sub(pc); //Vector Linepoint0 to Sphere

			//Sphere Ray intersection
			A = d.dot(d);
			B = (d.mult(2)).dot( p0pc);
			C = p0pc.dot(p0pc)-(radius*radius);
			D= (B*B) - (4.0*A*C); //Discriminant

			if (D>=0)
			{
				//mouse-ray hits the sphere

				//Calculate parameters of the two intersection Points
				float t1 = (- B + sqrt(D))/(2.0*A);
				float t2 = (- B - sqrt(D))/(2.0*A);

				//get euclidic position of selected node
				CVector3 curpos = m_pGeometry->ProjectToEuclid(m_pGraph->getPosition(m_iNearestnode));

				//rotate this position
				curpos.TransformByMatrix(m_pfRotationMatrix);

				float t;

				if (m_bFirstTimeDragging)
				{
					//if this is the first time we entered dragging mode,
					//we define on which side of the sphere the node is when
					//dragging continues
					if (curpos.z>0)
						m_bSphericalDragging_FrontBack = true;
					else
						m_bSphericalDragging_FrontBack = false;				
				}

				//get correct intersection point parameter, based on whether the node is in
				//front or back of the sphere.
				if (!m_bSphericalDragging_FrontBack)
					t = min(t2,t1); //get nearer one
				else
					t = max(t2,t1);

				//calculate intersection
				CVector3 Intersection = pd.add( d.mult(t) );

				//project intersection to from euclidic to spherical coordinates, set node position to 
				//this position.
				m_pGraph->SetPosition(m_iNearestnode, m_pGeometry->ProjectFromEuclid(Intersection), m_pGeometry);
			}
			else
			{
				//mouse-ray does not hit the sphere
                
				//set the position of the node to the point on the sphere that is nearest to 
				//the mouse cursor
				CVector3 mouseDirection = pd;
				mouseDirection.normalize();
				mouseDirection = mouseDirection.mult(radius);
				m_pGraph->SetPosition(m_iNearestnode, m_pGeometry->ProjectFromEuclid(mouseDirection), m_pGeometry);



			}




		}

		m_bFirstTimeDragging=false;

	}

	//Right Mouse button down - either Rotate or Hyperbolic Focusing
	if (m_rbuttondown)
	{
		
		if ( (m_iDisplayMode==eEuclidic3D) ||(m_iDisplayMode==eSpherical) )
		{
			//Rotate

			CVector3 nodePosition;

			//load current rotation matrix
			glPushMatrix();
			glLoadIdentity();
			glMultMatrixf(m_pfRotationMatrix);
			
			if ( (m_bSelected) & (m_iDisplayMode==eEuclidic3D))
			{
				//Rotate around selected node

                //get node position
				nodePosition = m_pGeometry->ProjectToEuclid(m_pGraph->getPosition(m_iNearestnode));	
				//translate rotation.center to node position
				glTranslatef(nodePosition.x,nodePosition.y,nodePosition.z);				
				m_rotateEnabled=true;
				glGetFloatv(GL_MODELVIEW_MATRIX,m_pfRotationMatrix);
			}

            //untransformed rotation axes			
			CVector3 y(0,1,0);
			CVector3 x(1,0,0);

			GLfloat temp[16];

			//rotate x-axis into current rotation-orthonormalbasis
			x.TransformByMatrix3Transpose(m_pfRotationMatrix);

			//rotate around this axis based on the mousemovement
			glRotatef(m_iDeltaMouseYRel*400.0, x.x,x.y,x.z);

			//get modelview matrix
			glGetFloatv(GL_MODELVIEW_MATRIX,temp);

			//rotate y-axis into current rotation-orthonormalbasis
			y.TransformByMatrix3Transpose(temp);

			//rotate around this axis based on the mousemovement
			glRotatef(m_iDeltaMouseXRel*400.0, y.x,y.y,y.z);

			//rotate around selected node?
			if ( (m_bSelected) & (m_iDisplayMode==eEuclidic3D))
				//translate rotation-center back to (0,0,0)
				glTranslatef(-nodePosition.x,-nodePosition.y,-nodePosition.z);

			glGetFloatv(GL_MODELVIEW_MATRIX,m_pfRotationMatrix);
			glPopMatrix();

		}

		if ( (m_iDisplayMode==eHyperbolic) & (!m_bHyperbolicDragMode) )
		{
			//Hyperbolic focusing - first click

			float pr = ((cHyperbolicGeometry*)m_pGeometry)->getPoincareRadius();

			//get viewspace position of mouseposition
			CVector3 mousepos (posx, posy,0); 		
			//get z of point(0,0,0)
			gluProject(0,0,0,modelMatrix, projMatrix, viewport,&X0, &Y0, &Z0);
			//use z of point (0,0,0) to get mousepos in the z=0 plane
			gluUnProject(posx, posy,Z0, modelMatrix, projMatrix, viewport,
							&X0, &Y0, &Z0);
			CVector3 mousePosWorld(X0,Y0,Z0);

			//mouse in poincare-disc?
			if (mousePosWorld.length() < pr)
			{
				//calculate poincare position of mouse
				//set new desired coa
				((cHyperbolicGeometry*)m_pGeometry)->setDesiredCenterOfAttention(complex<double> ( mousePosWorld.x/pr, mousePosWorld.y/pr));
				
				//enable drag-mode
				m_bHyperbolicDragMode=true;

			}
		}
		else
		if ( (m_iDisplayMode==eHyperbolic) & (m_bHyperbolicDragMode) )
		{
			//Hyperbolic focusing - drag-mode

			//get current desired coa
			complex<double> dcoa = ((cHyperbolicGeometry*)m_pGeometry)->getDesiredCenterOfAttention();
			
			//move desired coa based on mousemovement
			complex<double> move ( 2.0*(float)m_iDeltaMouseX/(float)m_Window.GetWidth(),
				-2.0*(float)m_iDeltaMouseY/(float)m_Window.GetHeight());
			
			//transform dcoa from center
			dcoa = (-move-dcoa)/( (-(conj(dcoa))*move)-1.0);
			
			//set new dcoa
			((cHyperbolicGeometry*)m_pGeometry)->setDesiredCenterOfAttentionDirect(dcoa);   

		}

	}
	else
	{
		m_bHyperbolicDragMode=false;
		m_rotateEnabled=false;
	}

			

	//For soft Focusing, force directed movement from current coa to desired coa
	if (m_iDisplayMode==eHyperbolic)
		((cHyperbolicGeometry*)m_pGeometry)->IntegrateCenterOfAttention((float)milliseconds/2000.0f);
	

	//Set Fog attenuation based on min/max z values from previous pass
	glFogf(GL_FOG_START,abs(m_fMaxz));
	glFogf(GL_FOG_END,abs(m_fMinz));

	//Set mouse-variables to current mouse position
	m_iDeltaMouseX =pos.x;
	m_iDeltaMouseY =pos.y;
	m_wheel_moved=false;

}

void GL_SpringEmbedders::OutPutStaticText(char* texfieldName, float n)
{
	
	using namespace CEGUI;
	char buffer[50];

	StaticText* st = (StaticText*) WindowManager::getSingleton().getWindow((utf8*)texfieldName);
	if (st)
	{
		_gcvt( n,  2, buffer );
		st->setText(buffer);
	}
}

void GL_SpringEmbedders::OutPutStaticText(char* texfieldName, char* text)
{
	using namespace CEGUI;

	StaticText* st = (StaticText*) WindowManager::getSingleton().getWindow((utf8*)texfieldName);

	if (st)
	{
		st->setText(text);
	}
}



GLvoid  GL_SpringEmbedders::BuildFont(int size)								// Build Our Bitmap Font
{
	//Font specific, From NeHe OpenGL Tutorials, by NeHe, http://nehe.gamedev.net/

	HFONT	font;										// Windows Font ID
	HFONT	oldfont;									// Used For Good House Keeping

	m_uiFontBase = glGenLists(96);								// Storage For 96 Characters

	font = CreateFont(	-size,							// Height Of Font
						0,								// Width Of Font
						0,								// Angle Of Escapement
						0,								// Orientation Angle
						FW_BOLD,						// Font Weight
						FALSE,							// Italic
						FALSE,							// Underline
						FALSE,							// Strikeout
						ANSI_CHARSET,					// Character Set Identifier
						OUT_TT_PRECIS,					// Output Precision
						CLIP_DEFAULT_PRECIS,			// Clipping Precision
						ANTIALIASED_QUALITY,			// Output Quality
						FF_DONTCARE|DEFAULT_PITCH,		// Family And Pitch
						NULL);					// Font Name Courier New

	oldfont = (HFONT)SelectObject(m_Window.getHDC(), font);           // Selects The Font We Want
	wglUseFontBitmaps(m_Window.getHDC(), 32, 96, m_uiFontBase);				// Builds 96 Characters Starting At Character 32
	SelectObject(m_Window.getHDC(), oldfont);							// Selects The Font We Want
	DeleteObject(font);									// Delete The Font
}

GLvoid  GL_SpringEmbedders::KillFont(GLvoid)									// Delete The Font List
{
	//Font specific, From NeHe OpenGL Tutorials, by NeHe, http://nehe.gamedev.net/


	glDeleteLists(m_uiFontBase, 96);							// Delete All 96 Characters
}

GLvoid  GL_SpringEmbedders::glPrint(const char *fmt, ...)					// Custom GL "Print" Routine
{
	//Font specific, From NeHe OpenGL Tutorials, by NeHe, http://nehe.gamedev.net/

	char		text[256];								// Holds Our String
	va_list		ap;										// Pointer To List Of Arguments

	if (fmt == NULL)									// If There's No Text
		return;											// Do Nothing

	va_start(ap, fmt);									// Parses The String For Variables
	    vsprintf(text, fmt, ap);						// And Converts Symbols To Actual Numbers
	va_end(ap);											// Results Are Stored In Text

	glPushAttrib(GL_LIST_BIT);							// Pushes The Display List Bits
	glListBase(m_uiFontBase - 32);								// Sets The Base Character to 32
	glCallLists(strlen(text), GL_UNSIGNED_BYTE, text);	// Draws The Display List Text
	glPopAttrib();										// Pops The Display List Bits


}


void GL_SpringEmbedders::Draw()											// Perform All Scene Drawing
{

	//reset min/maxz for correct min/maxz calculation
	m_fMaxz=-1000000;
	m_fMinz=1000000;

	//for current modelview matrix
	GLfloat transform[16];
	glGetFloatv(GL_MODELVIEW_MATRIX,transform);
	

	glClearColor(1,1,1,1);
	glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);		// Clear Screen And Depth Buffer

	using namespace CEGUI;

	Checkbox* chk;

	chk = (Checkbox*)WindowManager::getSingleton().getWindow((utf8*)"dp_chk1");
	//Graph-drawing enabled?
	if (chk->isSelected() )
	{
		//Draw Graph

		CVector3 Position, Position2 ;

		//Use fog if in euclidic-mode
		if (m_iDisplayMode == eEuclidic3D)
		{
			SetFogColor(m_vecColorFogMain);
			glEnable(GL_FOG);
		}

		//step trough every node
		for (unsigned int i=0; i< m_pGraph->getNumNodes(); i++)
		{
			//get euclidic position of node
			Position = m_pGeometry->ProjectToEuclid(m_pGraph->getPosition(i));

			glPushMatrix();

			

			//To Calculate max/minz in cameraspace:
			//Transform point to camera space
			CVector3 PosTransformed = Position;
			PosTransformed.TransformByMatrix(transform);
			//Test if point is clipped, quite a hack but it works...
			GLboolean clipped;
			glRasterPos3f(Position.x,Position.y,Position.z);
			//rasterpos is only valid if point is not clipped!
			glGetBooleanv(GL_CURRENT_RASTER_POSITION_VALID, &clipped);
            if (clipped)
			{
				//only use not-clipped points for min/maxz calculation
				if (m_fMaxz<PosTransformed.z)
					m_fMaxz=PosTransformed.z;
				if (m_fMinz>PosTransformed.z)
					m_fMinz=PosTransformed.z;
			}
			

			//translate to current euclidic positon
			glTranslatef(Position.x, Position.y, Position.z);
			
			//Set color based on selection
			if (m_pGraph->isSelected(i))
			{				
				SetColor(m_vecColorContrast1);
				SetFogColor(m_vecColorFogContrast1);

			}
			else
			{				
				SetColor(m_vecColorMain);
				SetFogColor(m_vecColorFogMain);
			}

			//Calculate scale of node, based on weight and maximum weight
			float scale = 0.5+( m_pGraph->getWeight(i)/m_pGraph->getMaxScale())*1.5f ;


			if (m_iDisplayMode == eHyperbolic)
			{
				//in hyperbolic mode, modify node scale proportional to distance from origin
				scale *= 1.0-1.0*( __max( (Position).length() / ((cHyperbolicGeometry*)m_pGeometry)->getPoincareRadius(),0));
				scale+=0.2;

			}


			//set scale
			glScalef(scale,scale,scale);
			//draw sphere
			glCallList(m_uiSphereDL);
			glScalef(1,1,1);
			glPopMatrix();


			chk = (Checkbox*)WindowManager::getSingleton().getWindow((utf8*)"dp_chk11");
			//draw lines?
			if (chk->isSelected())
			{
				//glDisable(GL_LIGHTING);

				//for all nodes following current node
				for (unsigned int ii=i+1; ii<m_pGraph->getNumNodes(); ii++)
				{
					//if one of those two nodes is selected: draw highlighted line
					if (m_pGraph->isSelected(i)|m_pGraph->isSelected(ii))
					{
						SetColor(m_vecColorContrast1);
					}
					else
					{
						SetColor(m_vecColorMainLight);
					}
				
					//if nodes are connected
					if ( m_pGraph->isConnected(i,ii) )
					{
						//get euclidic position of  node ii
						Position2 =  m_pGeometry->ProjectToEuclid(m_pGraph->getPosition(ii));
						
						if (m_pGeometry->isLinearInterpolable())
						{	
							//if linear-interpolable: just draw straight line
							glBegin(GL_LINES);
							glVertex3f(Position.x, Position.y, Position.z);
							glVertex3f(Position2.x, Position2.y, Position2.z);
							glEnd();
						}
						else
						{
							//draw geodesic (shortest path between points)
							CVector3 InterpolatedPosition;
							m_pGeometry->interpolateDraw(Position,Position2, 30);
						}
					}

				}				
			}
		}


		glDisable(GL_FOG);

	}


	chk = (Checkbox*)WindowManager::getSingleton().getWindow((utf8*)"dp_chk12");		
	//Should text be drawn
	if (chk->isSelected())
	{

		SetColor(m_vecColorContrast2);

		//calculate offset-vectors for text. The text should be moved in front of the node and be centered

		CVector3 z(0,0,5);
		//calculate z-Offset-vector in camera-space
		z.TransformByMatrix3Transpose(m_pfRotationMatrix);

		CVector3 x(-0.15,0,0);
		//calculate x-Offset-vector in camera-space
		x.TransformByMatrix3Transpose(m_pfRotationMatrix);

		chk = (Checkbox*)WindowManager::getSingleton().getWindow((utf8*)"dp_chk121");
		//draw just text of selected node?
		bool drawselected = (chk->isSelected());

		chk = (Checkbox*)WindowManager::getSingleton().getWindow((utf8*)"dp_chk1213");	
		//draw text of connected nodes too?
		bool drawconnected = (chk->isSelected());

		//step trough all nodes
		for (int i=0; i< m_pGraph->getNumNodes();i++)
		{
            //decide if text should be drawn based on user-selected drawingmode
			if ( ((m_iSelectedNode==i)& (m_bSelected)&(drawselected)) |			
				 (m_pGraph->isConnected(i, m_iSelectedNode))&(m_bSelected)&(drawconnected)|
				((!drawselected)&(!drawconnected) ))
			{}
			else
               continue;
						
			
			//get euclidic position of current node
			CVector3 Position =  m_pGeometry->ProjectToEuclid(m_pGraph->getPosition(i));

			//show toggles if text is viewed for this node based on view-dependent 
			//properties
			bool show=true;

			//only modify show if in "show all" mode
			if ( (!drawselected) & (!drawconnected) )
			{
				
				show=false;

				if (m_iDisplayMode==eSpherical)
				{
					//Do not show text in spherical mode if node is on the backside of the sphere
					CVector3 proj = Position;
					proj.TransformByMatrix3(m_pfRotationMatrix);
					if  (proj.z> ((cSphericalGeometry*)m_pGeometry)->getRadius()/3.0f)
						show=true;
				}

				if (m_iDisplayMode==eHyperbolic)
				{
					//do not show text in hyperbolic mode if text is near the border of the poincare disc
					CVector3 proj = Position;
					
					if (proj.length()< ((cHyperbolicGeometry*)m_pGeometry)->getPoincareRadius()/1.2f)
						show=true;
				}

				if (m_iDisplayMode==eEuclidic3D)
				{
					//always show in euclidic mode
					show=true;
				}
			}

			if (show)			
			{
				//draw text
				int length = m_pGraph->getNodeLabel(i).length();

				//offset position based on textlength. move text in front of the node.
				Position=(Position.add(z)).add(x.mult((float)length));
				glRasterPos3f(Position.x,Position.y,Position.z);	
				glPrint((m_pGraph->getNodeLabel(i)).data());

			}
		}



	}	

	CHK_GL_ERROR

	SetColor(m_vecColorMain);

	//draw coordinate system
	m_pGeometry->DrawAxis();

	CEGUI::System::getSingleton().renderGUI();
}



void GL_SpringEmbedders::ResetSpringconstantSliders()
{
	using namespace CEGUI;

	Slider* sl = (Slider*)WindowManager::getSingleton().getWindow((utf8*)"s1");
	sl->setCurrentValue(0.1);sl->setCurrentValue(0.2);
	sl = (Slider*)WindowManager::getSingleton().getWindow((utf8*)"s2");
	sl->setCurrentValue(0.1);sl->setCurrentValue(0.5);
	sl = (Slider*)WindowManager::getSingleton().getWindow((utf8*)"s3");
	sl->setCurrentValue(0.1);sl->setCurrentValue(0.5);

	//sl = (Slider*)WindowManager::getSingleton().getWindow((utf8*)"s31");
	//sl->setCurrentValue(0.1);sl->setCurrentValue(0.0);
}


//set m_pGeometry corresponding to the selected displaymode
//initialize m_pSpring and m_pGeometry to appropriate values for the selected geometry.
bool GL_SpringEmbedders::HandlerDisplaymode(const CEGUI::EventArgs& e)
{

	m_bNoTimeStep=true;

	using namespace CEGUI;

	RadioButton* rb = (RadioButton*)WindowManager::getSingleton().getWindow((utf8*)"rb1");
	if (rb->isSelected())
	{		
		free(m_pGeometry);


		m_pSpring->setSpringParametersBoundaries(0,5,1,40,0,150);
		ResetSpringconstantSliders();
		m_pSpring->setRepulsionCutoff(0.05f);

		//sl = (Slider*)WindowManager::getSingleton().getWindow((utf8*)"s31");
		//sl->setCurrentValue(0.1);sl->setCurrentValue(0.0);

		m_pGeometry = (cGeometry*) new cEuclidianGeometry;
		m_pGraph->setIs2DRepresentation(false);
		m_pGraph->setUsedPositionSet(1);

		if (m_bFirsttimeDisplaymode[0]==true)
		{
			m_pGraph->RandomizePositions(m_pGeometry);	
			m_bFirsttimeDisplaymode[0]=false;

		}
		
		m_iDisplayMode=eEuclidic3D;

		OutPutStaticText("descr1", "Middle Click: Move");
		OutPutStaticText("descr12", "Right Click: Rotate");
		OutPutStaticText("descr13", "Mousewheel: Zoom Camera");
		OutPutStaticText("descr14", "");

		((StaticText*) WindowManager::getSingleton().getWindow((utf8*)"Sphere Radius"))->setVisible(false);
		((Slider*)WindowManager::getSingleton().getWindow((utf8*)"sl_sphere"))->setVisible(false);

		return true;
	}

	rb = (RadioButton*)WindowManager::getSingleton().getWindow((utf8*)"rb3");
	if (rb->isSelected())
	{	
		//m_pGraph->setTimeStepMult(0.00001f);

		m_pSpring->setSpringParametersBoundaries(0,5,1,30,0,150);
		ResetSpringconstantSliders();
		m_pSpring->setRepulsionCutoff(0.05f);

		//sl = (Slider*)WindowManager::getSingleton().getWindow((utf8*)"s31");
		//sl->setCurrentValue(0.1);sl->setCurrentValue(0.0);

		free(m_pGeometry);
		m_pGeometry = (cGeometry*) new cSphericalGeometry;
		((cSphericalGeometry*)m_pGeometry)->setRadius(10);
		m_pGraph->setIs2DRepresentation(true);
		m_pGraph->setUsedPositionSet(2);
		if (m_bFirsttimeDisplaymode[1]==true)
		{
			m_pGraph->RandomizePositions(m_pGeometry);	
			m_bFirsttimeDisplaymode[1]=false;

		}
		
		m_iDisplayMode=eSpherical;

		OutPutStaticText("descr1", "Middle Click: Move");
		OutPutStaticText("descr12", "Right Click: Rotate");
		OutPutStaticText("descr13", "Mousewheel: Zoom Camera");
		OutPutStaticText("descr14", "");
		((StaticText*) WindowManager::getSingleton().getWindow((utf8*)"Sphere Radius"))->setEnabled(true);
		((Slider*)WindowManager::getSingleton().getWindow((utf8*)"sl_sphere"))->setEnabled(true);
		((StaticText*) WindowManager::getSingleton().getWindow((utf8*)"Sphere Radius"))->setVisible(true);
		((Slider*)WindowManager::getSingleton().getWindow((utf8*)"sl_sphere"))->setVisible(true);

		Slider* sl = (Slider*)WindowManager::getSingleton().getWindow((utf8*)"sl_sphere");
		sl->subscribeEvent(Slider::EventValueChanged,CEGUI::Event::Subscriber(&GL_SpringEmbedders::					HandlerSphereSize,this));

		sl->setCurrentValue(0.2);
		sl->setCurrentValue(0.35);

		return true;
	}

	rb = (RadioButton*)WindowManager::getSingleton().getWindow((utf8*)"rb2");
	if (rb->isSelected())
	{
		
		free(m_pGeometry);

		m_pSpring->setSpringParametersBoundaries(0,5,0,3,0,10);
		ResetSpringconstantSliders();
		m_pSpring->setRepulsionCutoff(0.5f);

		//sl = (Slider*)WindowManager::getSingleton().getWindow((utf8*)"s31");
		//sl->setCurrentValue(0.1);sl->setCurrentValue(0.0);

		m_pGeometry = (cGeometry*) new cHyperbolicGeometry;
		m_pGraph->setIs2DRepresentation(true);
		m_pGraph->setUsedPositionSet(3);
		if (m_bFirsttimeDisplaymode[2]==true)
		{
			m_pGraph->RandomizePositions(m_pGeometry);	
			m_bFirsttimeDisplaymode[2]=false;

		}
		
		m_iDisplayMode=eHyperbolic;

		glPushMatrix();
		glLoadIdentity();
		glGetFloatv(GL_MODELVIEW_MATRIX,m_pfRotationMatrix);
		glPopMatrix();

		OutPutStaticText("descr1", "Middle Click: Move");
		OutPutStaticText("descr12", "Right Click: Focus");
		OutPutStaticText("descr13", "Right Drag: Move Focus");
		OutPutStaticText("descr14", "Mousewheel: Zoom Camera");
		((StaticText*) WindowManager::getSingleton().getWindow((utf8*)"Sphere Radius"))->setVisible(false);
		((Slider*)WindowManager::getSingleton().getWindow((utf8*)"sl_sphere"))->setVisible(false);
		return true;
	}





	return false;

}

bool GL_SpringEmbedders::HandlerExit(const CEGUI::EventArgs& e)
{
	TerminateApplication();
	return true;
}

bool GL_SpringEmbedders::HandlerExitWindow(const CEGUI::EventArgs& e)
{
	using namespace CEGUI;
	FrameWindow* fw = (FrameWindow*)WindowManager::getSingleton().getWindow((utf8*)"Exit");
	fw->setVisible(true);
	return true;
}


bool GL_SpringEmbedders::HandlerExitNo(const CEGUI::EventArgs& e)
{
	using namespace CEGUI;
	FrameWindow* fw = (FrameWindow*)WindowManager::getSingleton().getWindow((utf8*)"Exit");
	fw->setVisible(false);
	return true;
}


bool GL_SpringEmbedders::HandlerSpringparameters(const CEGUI::EventArgs& e)
{

	using namespace CEGUI;

	Slider* sl;
	float value1, value2;

	sl = (Slider*)WindowManager::getSingleton().getWindow((utf8*)"s1");
	value1 = sl->getCurrentValue();
	sl = (Slider*)WindowManager::getSingleton().getWindow((utf8*)"s2");
	value2 = sl->getCurrentValue();

	m_pSpring->setSpringParametersRelative(value2, value1);

	OutPutStaticText("st_v1", m_pSpring->getSpringLength());
	OutPutStaticText("st_v11", m_pSpring->getSpringConstant());

	sl = (Slider*)WindowManager::getSingleton().getWindow((utf8*)"s3");	
	value1 = sl->getCurrentValue();
	m_pSpring->setRepulsionStrengthRelative(value1);

	OutPutStaticText("st_v12", m_pSpring->getRepulsionStrength());

	sl = (Slider*)WindowManager::getSingleton().getWindow((utf8*)"s31");	
	value1 = sl->getCurrentValue()*80;
	m_pGraph->setTimeStepMult(value1);

	if (FLOAT_EQ(value1,0))
	{
		OutPutStaticText("st_v121","sim. off !");	
		StaticText* st = (StaticText*) WindowManager::getSingleton().getWindow((utf8*)"st_v121");
		st->setFrameColours( ColourRect(colour( 1,0,0,1 )));
		st->setFrameEnabled(true);
	


	}
	else
	{
		OutPutStaticText("st_v121", value1);	
		StaticText* st = (StaticText*) WindowManager::getSingleton().getWindow((utf8*)"st_v121");
		st->setFrameColours( ColourRect(colour( 1,0,0,1 )));
		st->setFrameEnabled(false);
	}

	return true;

}

bool GL_SpringEmbedders::HandlerSphereSize(const CEGUI::EventArgs& e)
{
	using namespace CEGUI;
	
	Slider* sl;
	sl = (Slider*)WindowManager::getSingleton().getWindow((utf8*)"sl_sphere");
	float value = sl->getCurrentValue()*30+6;

	if (m_iDisplayMode==eSpherical)
		((cSphericalGeometry*)m_pGeometry)->setRadius(value);

	return true;
}

bool GL_SpringEmbedders::HandlerResetCamera(const CEGUI::EventArgs& e)
{
	glPushMatrix();
	glLoadIdentity();
	glGetFloatv(GL_MODELVIEW_MATRIX,m_pfRotationMatrix);
	glPopMatrix();

	m_vecCameraDelta.Set(0,0,0);


	return true;
}

bool GL_SpringEmbedders::HandlerLoadGraph(const CEGUI::EventArgs& e)
{
	using namespace CEGUI;

	m_bXGMMLSelected=false;

	FrameWindow* fw = (FrameWindow*)WindowManager::getSingleton().getWindow((utf8*)"Load");
	fw->setVisible(true);	
	m_sDirectory= string(".\\");

	LoadListbox();

	return true;
}


void GL_SpringEmbedders::LoadListbox()
{
	//For load-file dialog
	using namespace CEGUI;
	Listbox* lb = (Listbox*)WindowManager::getSingleton().getWindow((utf8*)"load_lb1");
	
	lb->setSortingEnabled(true);
	lb->resetList();

    struct _finddata_t c_file;
    long hFile;

	string file = m_sDirectory;
	file.append("*");

	if( (hFile = _findfirst( (file).data(), &c_file )) == -1L )
	{

	}
	else
	{
		bool* dir = new bool;
		CEGUI::ListboxTextItem* item = new CEGUI::ListboxTextItem( c_file.name);
		item->setSelectionBrushImage("WindowsLook", "Background");
		if (c_file.attrib & _A_SUBDIR )
		{
	
			item->setTextColours( CEGUI::colour(0.0f,0.0f,1.0f) );
			*dir=true;

		}
		else
		{
			item->setTextColours( CEGUI::colour(1.0f,0.0f,0.0f) );
			*dir=false;
		}
		item->setUserData((void*)dir);
		if ( (*dir) || ( strstr(c_file.name, ".xgmml")))
			lb->addItem(item);

        while( _findnext( hFile, &c_file ) == 0 )
        {
	
			bool* dir = new bool;
			CEGUI::ListboxTextItem* item = new CEGUI::ListboxTextItem( c_file.name);
			item->setSelectionBrushImage("WindowsLook", "Background");
			if (c_file.attrib & _A_SUBDIR )
			{
				item->setTextColours( CEGUI::colour(0.0f,0.0f,1.0f) );
				*dir=true;

			}
			else
			{
				item->setTextColours( CEGUI::colour(1.0f,0.0f,0.0f) );
				*dir=false;
			}
			item->setUserData((void*)dir);
			if ( (*dir) || ( strstr(c_file.name, ".xgmml")))
				lb->addItem(item);

        }
       _findclose( hFile );
   }

}

bool GL_SpringEmbedders::HandlerLoadCancel(const CEGUI::EventArgs& e)
{
	using namespace CEGUI;
	FrameWindow* fw = (FrameWindow*)WindowManager::getSingleton().getWindow((utf8*)"Load");
	fw->setVisible(false);
	Listbox* lb = (Listbox*)WindowManager::getSingleton().getWindow((utf8*)"load_lb1");
	lb->resetList();
	return true;
}

bool GL_SpringEmbedders::HandlerLoadOK(const CEGUI::EventArgs& e)
{
	using namespace CEGUI;


	Checkbox* chk;

	chk = (Checkbox*)WindowManager::getSingleton().getWindow((utf8*)"load_cb1");	

	errHandler->resetErrors();

	if (chk->isSelected())
		parser->setValidationScheme(XercesDOMParser::Val_Always); 
	else
		parser->setValidationScheme(XercesDOMParser::Val_Never); 

	if (m_bXGMMLSelected)
	{
		Editbox* eb = (Editbox*)WindowManager::getSingleton().getWindow((utf8*)"load_eb1");	
		string filename = string((char*)eb->getText().data());
		m_sDirectory.append(filename);

		m_pGraph->loadXMLGraph((char*)m_sDirectory.data(),parser, errHandler);
		
		m_bFirsttimeDisplaymode[0]=true;
		m_bFirsttimeDisplaymode[1]=true;
		m_bFirsttimeDisplaymode[2]=true;

		m_bNoTimeStep=true;

		Slider* sl = (Slider*)WindowManager::getSingleton().getWindow((utf8*)"s31");
		sl->setCurrentValue(0.1);sl->setCurrentValue(0.0);
	}

	HandlerDisplaymode(EventArgs());
	FrameWindow* fw = (FrameWindow*)WindowManager::getSingleton().getWindow((utf8*)"Load");
	fw->setVisible(false);
	Listbox* lb = (Listbox*)WindowManager::getSingleton().getWindow((utf8*)"load_lb1");
	lb->resetList();
	return true;
}

bool GL_SpringEmbedders::HandlerLoadBrowse(const CEGUI::EventArgs& e)
{
	using namespace CEGUI;

	Listbox* lb = (Listbox*)WindowManager::getSingleton().getWindow((utf8*)"load_lb1");

	ListboxItem* item = lb->getFirstSelectedItem();	
	bool dir = *((bool*)item->getUserData());

	if (dir)
	{
		//m_sDirectory = (string((char*)item->getText().data()).append("\\") ).append(m_sDirectory);

		m_sDirectory = m_sDirectory.append(string((char*)item->getText().data())).append("\\");
		LoadListbox();
		Editbox* eb = (Editbox*)WindowManager::getSingleton().getWindow((utf8*)"load_eb1");
		eb->setText("");
		m_bXGMMLSelected=false;
	}
	else
	{
		Editbox* eb = (Editbox*)WindowManager::getSingleton().getWindow((utf8*)"load_eb1");
		eb->setText(item->getText());
		m_bXGMMLSelected=true;
	}

	return true;

}

bool GL_SpringEmbedders::HandlerLoadCB1(const CEGUI::EventArgs& e)
{
	using namespace CEGUI;
	Checkbox* chk;	
	chk = (Checkbox*)WindowManager::getSingleton().getWindow((utf8*)"load_cb1");
	m_bStrictParsing=chk->isSelected();
	return true;
}


bool GL_SpringEmbedders::HandlerFontsize(const CEGUI::EventArgs& e)
{
	KillFont();
	using namespace CEGUI;
	Slider* sl = (Slider*)WindowManager::getSingleton().getWindow((utf8*)"dp_sl1");
	int size = (int) (sl->getCurrentValue()+5.0);
	BuildFont(size);
	return true;
}


bool GL_SpringEmbedders::HandlerReset(const CEGUI::EventArgs& e)
{
	m_pGraph->RandomizePositions(m_pGeometry)  ;
	m_bNoTimeStep=true;

	return true;

}



void GL_SpringEmbedders::createInterface()
{

	//Initialize Interface

	using namespace CEGUI;
	try{
		Logger::getSingleton().setLoggingLevel(Informative);

		if(!SchemeManager::getSingletonPtr()->isSchemePresent((utf8*)"WindowsLook"))
			SchemeManager::getSingletonPtr()->loadScheme((utf8*)"WindowsLook.scheme");

		System::getSingletonPtr()->setDefaultMouseCursor((utf8*)"WindowsLook", (utf8*)"MouseArrow");

		CEGUI::Font *font;
		if(!CEGUI::FontManager::getSingleton().isFontPresent("tahoma-12"))
		{
			font = CEGUI::FontManager::getSingleton().createFont( ".\\guidata\\tahoma-12.font" );
		}
		else
		{
			font=CEGUI::FontManager::getSingleton().getFont("tahoma-12");
		}
		
		CEGUI::System::getSingleton().setDefaultFont( font );

		
		Window* myRoot = WindowManager::getSingleton().loadWindowLayout(".\\guidata\\gui.xml");
		System::getSingleton().setGUISheet(myRoot);
	


	}
	catch(CEGUI::Exception &c){}
	{
		//no gui will be drawn
	}

	//set parameters, subscribe to handlers
	FrameWindow* fw = (FrameWindow*)WindowManager::getSingleton().getWindow((utf8*)"controll");
	fw->subscribeEvent(FrameWindow::EventCloseClicked, CEGUI::Event::Subscriber(&GL_SpringEmbedders::HandlerExitWindow,this));
	fw = (FrameWindow*)WindowManager::getSingleton().getWindow((utf8*)"Exit");
	fw->setVisible(false);

	fw = (FrameWindow*)WindowManager::getSingleton().getWindow((utf8*)"Load");
	fw->subscribeEvent(FrameWindow::EventCloseClicked, CEGUI::Event::Subscriber(&GL_SpringEmbedders::HandlerLoadCancel,this));
	fw->setVisible(false);

	fw = (FrameWindow*)WindowManager::getSingleton().getWindow((utf8*)"dp");
	fw->toggleRollup();

	RadioButton* rb = (RadioButton*)WindowManager::getSingleton().getWindow((utf8*)"rb1");
	rb->subscribeEvent(RadioButton::EventSelectStateChanged, CEGUI::Event::Subscriber(&GL_SpringEmbedders::HandlerDisplaymode,this));
	rb = (RadioButton*)WindowManager::getSingleton().getWindow((utf8*)"rb2");
	rb->subscribeEvent(RadioButton::EventSelectStateChanged, CEGUI::Event::Subscriber(&GL_SpringEmbedders::HandlerDisplaymode,this));
	rb->setSelected(true);
	rb = (RadioButton*)WindowManager::getSingleton().getWindow((utf8*)"rb3");
	rb->subscribeEvent(RadioButton::EventSelectStateChanged,CEGUI::Event::Subscriber(&GL_SpringEmbedders::HandlerDisplaymode,this));
	

	PushButton* b = (PushButton*)WindowManager::getSingleton().getWindow((utf8*)"b2");
	b->subscribeEvent(PushButton::EventClicked, CEGUI::Event::Subscriber(&GL_SpringEmbedders::HandlerReset,this));

	b = (PushButton*)WindowManager::getSingleton().getWindow((utf8*)"exit_btn_yes");
	b->subscribeEvent(PushButton::EventClicked, CEGUI::Event::Subscriber(&GL_SpringEmbedders::HandlerExit,this));

	b = (PushButton*)WindowManager::getSingleton().getWindow((utf8*)"exit_btn_yes3");
	b->subscribeEvent(PushButton::EventClicked, CEGUI::Event::Subscriber(&GL_SpringEmbedders::HandlerExitNo,this));

	b = (PushButton*)WindowManager::getSingleton().getWindow((utf8*)"b22");
	b->subscribeEvent(PushButton::EventClicked, CEGUI::Event::Subscriber(&GL_SpringEmbedders::HandlerResetCamera,this));

	b = (PushButton*)WindowManager::getSingleton().getWindow((utf8*)"b1");
	b->subscribeEvent(PushButton::EventClicked, CEGUI::Event::Subscriber(&GL_SpringEmbedders::HandlerLoadGraph,this));

	b = (PushButton*)WindowManager::getSingleton().getWindow((utf8*)"load_pb1");
	b->subscribeEvent(PushButton::EventClicked, CEGUI::Event::Subscriber(&GL_SpringEmbedders::HandlerLoadOK,this));
	b = (PushButton*)WindowManager::getSingleton().getWindow((utf8*)"load_pb2");
	b->subscribeEvent(PushButton::EventClicked, CEGUI::Event::Subscriber(&GL_SpringEmbedders::HandlerLoadCancel,this));

	Slider* sl = (Slider*)WindowManager::getSingleton().getWindow((utf8*)"s1");
	sl->subscribeEvent(Slider::EventValueChanged,CEGUI::Event::Subscriber(&GL_SpringEmbedders::HandlerSpringparameters,this));
	sl = (Slider*)WindowManager::getSingleton().getWindow((utf8*)"s2");
	sl->subscribeEvent(Slider::EventValueChanged,CEGUI::Event::Subscriber(&GL_SpringEmbedders::HandlerSpringparameters,this));
	sl = (Slider*)WindowManager::getSingleton().getWindow((utf8*)"s3");
	sl->subscribeEvent(Slider::EventValueChanged,CEGUI::Event::Subscriber(&GL_SpringEmbedders::HandlerSpringparameters,this));
	sl = (Slider*)WindowManager::getSingleton().getWindow((utf8*)"s31");
	sl->subscribeEvent(Slider::EventValueChanged,CEGUI::Event::Subscriber(&GL_SpringEmbedders::HandlerSpringparameters,this));

	sl = (Slider*)WindowManager::getSingleton().getWindow((utf8*)"dp_sl1");
	sl->subscribeEvent(Slider::EventValueChanged,CEGUI::Event::Subscriber(&GL_SpringEmbedders::HandlerFontsize,this));
	sl->setCurrentValue(10.0);
	

	StaticText* st = (StaticText*) WindowManager::getSingleton().getWindow((utf8*)"st1");
	st->setBackgroundEnabled(false);st->setFrameEnabled(false);
	st = (StaticText*) WindowManager::getSingleton().getWindow((utf8*)"st2");
	st->setBackgroundEnabled(false);st->setFrameEnabled(false);
	st = (StaticText*) WindowManager::getSingleton().getWindow((utf8*)"st3");
	st->setBackgroundEnabled(false);st->setFrameEnabled(false);
	//st = (StaticText*) WindowManager::getSingleton().getWindow((utf8*)"st4");
	//st->setBackgroundEnabled(false);st->setFrameEnabled(false);
	st = (StaticText*) WindowManager::getSingleton().getWindow((utf8*)"st5");
	st->setBackgroundEnabled(false);st->setFrameEnabled(false);

	st = (StaticText*) WindowManager::getSingleton().getWindow((utf8*)"descr1");
	st->setBackgroundEnabled(false);st->setFrameEnabled(false);
	st = (StaticText*) WindowManager::getSingleton().getWindow((utf8*)"descr12");
	st->setBackgroundEnabled(false);st->setFrameEnabled(false);
	st = (StaticText*) WindowManager::getSingleton().getWindow((utf8*)"descr13");
	st->setBackgroundEnabled(false);st->setFrameEnabled(false);
	st = (StaticText*) WindowManager::getSingleton().getWindow((utf8*)"descr14");
	st->setBackgroundEnabled(false);st->setFrameEnabled(false);
	st = (StaticText*) WindowManager::getSingleton().getWindow((utf8*)"Sphere Radius");
	st->setBackgroundEnabled(false);st->setFrameEnabled(false);
	st = (StaticText*) WindowManager::getSingleton().getWindow((utf8*)"st_exit");
	st->setBackgroundEnabled(false);st->setFrameEnabled(false);

	st = (StaticText*) WindowManager::getSingleton().getWindow((utf8*)"load_st1");
	st->setBackgroundEnabled(false);st->setFrameEnabled(false);

	st = (StaticText*) WindowManager::getSingleton().getWindow((utf8*)"dp_st1");
	st->setBackgroundEnabled(false);st->setFrameEnabled(false);

	st = (StaticText*) WindowManager::getSingleton().getWindow((utf8*)"st31");
	st->setBackgroundEnabled(false);st->setFrameEnabled(false);

	st = (StaticText*) WindowManager::getSingleton().getWindow((utf8*)"dp_st13");
	st->setBackgroundEnabled(false);st->setFrameEnabled(false);

	st = (StaticText*) WindowManager::getSingleton().getWindow((utf8*)"dp_st11");
	st->setBackgroundEnabled(false);st->setFrameEnabled(false);

	sl = (Slider*)WindowManager::getSingleton().getWindow((utf8*)"sl_sphere");
	sl->subscribeEvent(Slider::EventValueChanged,CEGUI::Event::Subscriber(&GL_SpringEmbedders::HandlerSphereSize,this));

	sl->setCurrentValue(0.3);

	ResetSpringconstantSliders();


	Checkbox* chk;	
	chk = (Checkbox*)WindowManager::getSingleton().getWindow((utf8*)"load_cb1");	
	chk->subscribeEvent(Checkbox::EventCheckStateChanged, CEGUI::Event::Subscriber(&GL_SpringEmbedders::HandlerLoadCB1,this));

	Listbox* lb = (Listbox*)WindowManager::getSingleton().getWindow((utf8*)"load_lb1");
	lb->subscribeEvent(Listbox::EventSelectionChanged, CEGUI::Event::Subscriber(&GL_SpringEmbedders::HandlerLoadBrowse,this));

	sl = (Slider*)WindowManager::getSingleton().getWindow((utf8*)"s31");
	sl->setCurrentValue(0.1);sl->setCurrentValue(0.0);
}


void GL_SpringEmbedders::initCEGUI()
{
	// create OpenGL GUI Renderer
	const unsigned int max_quads = 1000;
	const int width = m_Window.GetWidth(); // replace with your screen width function
	const int height = m_Window.GetHeight(); // replace with your screen height function
	m_GLrenderer = new CEGUI::OpenGLRenderer( max_quads, width, height );

	// create cegui system
	new CEGUI::System( m_GLrenderer );
	
	
	// load schemes
	CEGUI::SchemeManager::getSingleton().loadScheme( ".\\guidata\\WindowsLook.scheme" );

	
	// set default mouse cursor
	CEGUI::System::getSingleton().setDefaultMouseCursor( "WindowsLook", "MouseArrow" );
	
	
	// create the interface
	createInterface();

}

