/*! \mainpage SpringEmbedders - A program to visualize graphs in non-euclidean geometries using a spring-embedder Algorithm

\section a Copyright Notes
Copyright (C) 2005  Markus Lipp

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.


\section b SpringEmbedders uses following external components:

Crazy Eddie's GUI System Mk-2
http://www.cegui.org.uk
Copyright 2004 - 2005 Paul D Turner & Contributing Authors
"Crazy Eddie's GUI System Mk-2 is a free library providing windowing and
widgets for graphics APIs and engines where such functionality is not natively
available or is severely lacking." 
Licensed under "GNU LESSER GENERAL PUBLIC LICENSE", see "GNU LESSER GENERAL PUBLIC LICENSE.txt"

Xerces-C++ Version 2.6.0
http://xml.apache.org/xerces-c/
Licensed under "Apache Software License, Version 2.0.", see "ApacheLicense.txt".

Nehe OpenGL Framework, Adapted to C++ by Andreas Oberdorfer, 2004;
Nehe OpenGL Lesson 12, "Bitmap Fonts" by NeHe.
By Nehe: "The lessons on this page may contain mistakes, poor commenting, and should not be considered the best resource to learn OpenGL from. What you do with the code is up to you. I am merely trying to make the learning process a little easier for those people new to OpenGL."
*/



#ifndef __SPRING_H__
#define __SPRING_H__


#include "NeHeGL.h"												// Header File For NeHeGL


#include <gl\glaux.h>	
#include "glut.h"
#include <iostream>


#include "cGraph.h"
#include "cGeometry.h"
#include "cSpring.h"
#include "Vector3.h"


//#include <gl\gl.h>												// Header File For The OpenGL32 Library
//#include <gl\glu.h>												// Header File For The GLu32 Library
										// Header File For The GLaux Library
#include <stdlib.h>
#include <stdio.h>
#include <fstream>
#include <sys\stat.h>
#include <iostream>
#include <stdio.h>
#include <string.h>
#include <xercesc/dom/DOM.hpp>
#include <xercesc/util/PlatformUtils.hpp>
#include <xercesc/util/XMLString.hpp>
#include <iostream>
#include <xercesc/framework/StdOutFormatTarget.hpp>
#include <xercesc/dom/impl/DOMWriterImpl.hpp>
#include <xercesc/parsers/XercesDOMParser.hpp>
#include <xercesc/framework/LocalFileInputSource.hpp>
#include <xercesc/dom/DOMErrorHandler.hpp>
 #include <xercesc/sax/HandlerBase.hpp>
#include "DOMTreeErrorReporter.hpp"

#ifdef XERCES_CPP_NAMESPACE_USE
XERCES_CPP_NAMESPACE_USE
#endif


/** Main Springembedders-Application class*/
class GL_SpringEmbedders : GL_Application								// Derive From Application
{
public:
	bool	Initialize();										// Perform All Initialization
	void	Deinitialize();										// Perform All DeInitialization
	void	Update(DWORD milliseconds);							// Perform Motion Updates
	void	Draw();												// Perform All Scene Drawing
	

private:

	/** Initialize Xerces*/
	void initXerces();
	/** Xerces XML Parser*/
	XercesDOMParser *parser;
	/** Xerces Error handler*/
	DOMTreeErrorReporter* errHandler;

	/**Possible Display-modes*/
	enum DisplayMode {eEuclidic3D, eHyperbolic, eSpherical};
	/**Current Displaymode. Needed for Interaction and Visualization.*/
	int  m_iDisplayMode;

	/**output function for static GUI textfields*/ 
	void OutPutStaticText(char* texfieldName, float n);
	/**output function for static GUI textfields*/
	void OutPutStaticText(char* texfieldName, char* text);


	/**Sphere Displaylist */
	GLuint m_uiSphereDL;

	friend class GL_Application;
	GL_SpringEmbedders(const char * class_name); // Class Constructor


	/** Pointer to instance of cGraph*/
	cGraph*		m_pGraph;
	/** Pointer to instance of cGeometry*/
	cGeometry*  m_pGeometry;
	/** Pointer to instance of cSpring*/
	cSpring*	m_pSpring;

	/** Create GUI Interface, Load Layout, Initialize Message Handlers*/
	void createInterface();
	/** Initialize CrazyEddies GUI System*/
	void initCEGUI();

	//GUI handlers
	bool HandlerDisplaymode(const CEGUI::EventArgs& e);
	bool HandlerSpringparameters(const CEGUI::EventArgs& e);
	bool HandlerReset(const CEGUI::EventArgs& e);
	bool HandlerSphereSize(const CEGUI::EventArgs& e);
	bool HandlerResetCamera(const CEGUI::EventArgs& e);
	bool HandlerExit(const CEGUI::EventArgs& e);
	bool HandlerExitNo(const CEGUI::EventArgs& e);
	bool HandlerExitWindow(const CEGUI::EventArgs& e);
	bool HandlerLoadGraph(const CEGUI::EventArgs& e);
	bool HandlerLoadCancel(const CEGUI::EventArgs& e);
	bool HandlerLoadOK(const CEGUI::EventArgs& e);
	bool HandlerLoadBrowse(const CEGUI::EventArgs& e);
	bool HandlerLoadCB1(const CEGUI::EventArgs& e);
	bool HandlerFontsize(const CEGUI::EventArgs& e);	
	/**Input contents directory into listbox.*/
	void LoadListbox();
	void ResetSpringconstantSliders();

	/**Set OpenGL Ambient Material Color*/
	void SetAmbientMaterialColor(CVector3 color);
	/**Set OpenGL Color*/
	void SetColor(CVector3 color);
	/**Set OpenGL Fog-Color*/
	void SetFogColor(CVector3 color);

	
	//For Load-Screen. As Load-Screen is not modal, member variables are required.
	/**strict parsing enabled?*/
	bool m_bStrictParsing;
	/**Is an XGMML file selected in the load-screen*/
	bool m_bXGMMLSelected; 
	/**current directory of load-screen*/
	string m_sDirectory;


	//Interaction specific, Interaction logic

	/**Relative Mouse Movement*/
	int m_iDeltaMouseX, m_iDeltaMouseY;
	/**Rotation enabled?*/
	bool m_rotateEnabled;
	/**Nearest selected Node*/
	int	 m_iNearestnode;
	/**Is any Node selected?*/
	bool m_bSelected;
	/**Current selected Node*/
	int m_iSelectedNode;
	/**Is dragging (Moving of Nodes) enabled?*/
	bool m_dragEnabled;
	/**Is it the first time we are in drag mode while going trough update?*/
	bool m_bFirstTimeDragging;
	/**Should we use front or backside while dragging in spherical mode?*/
	bool m_bSphericalDragging_FrontBack;	
	/**Is it the first time we switch to a specific Displaymode?*/
	bool m_bFirsttimeDisplaymode[3];
	/**Are we in Hyperbolic Drag mode?*/
	bool m_bHyperbolicDragMode;


	/**Current Rotation Matrix */
	GLfloat m_pfRotationMatrix[16];
	/**Camera Offset*/
	CVector3 m_vecCameraDelta;
	/**min/max Depth-Values of current scene, needed for dynamic linear fog
	used in euclidic representation*/
	float m_fMaxz,m_fMinz;

	/**Disable Integration for the next frame?
	Necessary to avoid large timesteps during operations like loading.*/
	bool m_bNoTimeStep;

	/**Needed for Iterations/Second calculation*/
	int m_iElapsedtime;
	/**Needed for Iterations/Second calculation*/
	int iterationscount;

	//Font specific, From NeHe OpenGL Tutorials Lesson 12, by NeHe, http://nehe.gamedev.net/
	GLvoid  BuildFont(int size)	;
	GLvoid  KillFont(GLvoid);
	void	glPrint(const char *fmt, ...)			;
	/** Base Display List For The Font Set*/
	GLuint	m_uiFontBase;				

	//Color Profile Definitions
	CVector3 m_vecColorMain, m_vecColorBackground, m_vecColorContrast1, m_vecColorContrast2,
		m_vecColorFogMain,m_vecColorFogContrast1, m_vecColorFogContrast2,
		m_vecColorMainLight, m_vecColorContrast1Light, m_vecColorContrast2Light;
};

#endif