/*SpringEmbedders - A program to visualize graphs
in non-euclidean geometries using a Springembedder Algorithm

Copyright (C) 2005  Markus Lipp

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

/*

This file is based on:

/********************************************
* Object Orientated NeHeGL Using Base Class *
* Author: Andreas Oberdorfer           2004 *
********************************************

It was modified by Markus Lipp, 10.06.2005

Copyright Notice by Nehe: "The lessons on this page may contain mistakes, poor commenting, and should not be considered the best resource to learn OpenGL from. What you do with the code is up to you. I am merely trying to make the learning process a little easier for those people new to OpenGL."

*/


#ifndef	__GL_FRAMEWORK_H__
#define	__GL_FRAMEWORK_H__

/** Check for OpenGL Error*/
#define CHK_GL_ERROR  { GLenum error=glGetError(); if ( error != GL_NO_ERROR) { MessageBox(NULL,(LPCSTR) gluErrorString(error), "gl ERROR",MB_OK);};};
/** Check for MS Error*/
#define CHK_MS_ERROR  { DWORD errorValue = GetLastError(); if ( errorValue != NULL) MessageBox(NULL,(LPCSTR) GetErrorStringMS(errorValue), "ms ERROR",MB_OK);};


#include <glew.h>
#include <wglew.h>

#undef min
#undef max
#include "..\include\renderers\OpenGLGUIRenderer\openglrenderer.h"
#include "..\include\CEGUI.h"
#include <CEGUIGUILayout_xmlHandler.h>//XML loading

#include "GLWindow.h"											// Header File For GLWindow



/** This class encapsulates the state of the Keyboard*/
class Keys														// Class For Keyboard Stuff
{
public:
	// Class Constructor
	Keys() { Clear(); }
	// Clear All Keys
	void Clear() { ZeroMemory(&m_KeyDown, sizeof(m_KeyDown)); }
	// Return true If The Key State Is Pressed
	bool IsPressed(unsigned int key)	{ return (key < MAX_KEYS) ? (m_KeyDown[key] == true) : false; }
	// Set a Key To State Pressed
	void SetPressed(unsigned int key)	{ if (key < MAX_KEYS) m_KeyDown[key] = true; }
	// Set a Key To State Released
	void SetReleased(unsigned int key)	{ if (key < MAX_KEYS) m_KeyDown[key] = false; }

private:
	static const unsigned int MAX_KEYS = 256;
	bool m_KeyDown[MAX_KEYS];									// Holds true / false For Each Key
};


/** Base OpenGL Framework Class*/
class GL_Application										
{
public:

	/**Create Your Derived Class*/
	static GL_Application * Create(const char * class_name);	

	/** Virtal Destructor Is Necessary To Avoid Memory Leaks When Deleting The (Derived) Class*/
	virtual ~GL_Application() {};

protected:
	// These Methods Must Be Provided By Derived Class
	/** Perform All Initialization*/
	virtual bool	Initialize() = 0;	
	/** Perform All DeInitialization*/
	virtual void	Deinitialize() = 0;		
	/** Perform Motion Updates*/
	virtual void	Update(DWORD milliseconds) = 0;	
	/** Perform All Scene Drawing*/
	virtual void	Draw() = 0;									

	/** initialize CrazyEddieds GUI System */
	virtual void initCEGUI() {};

	// Common Methods Provided By Base Class
	/** Toggle Fullscreen / Windowed Mode*/
	void	ToggleFullscreen();									
	/** Terminate The Application*/
	void	TerminateApplication();								
	/** Enable Redraw During Window Resize*/
	void	ResizeDraw(bool enable) { m_ResizeDraw = enable; }	
	/** Key Class*/
	Keys	m_Keys;												
	

	/** Right Mouskey State*/
	bool	m_rbuttonup;
	/** Right Mouskey State*/
	bool	m_rbuttondown;
	/** Middle Mouskey State*/
	bool	m_mbuttonup;
	/** Right Mouskey State*/
	bool	m_mbuttondown;
	/** Left Mouskey State*/
    bool	m_lbuttonup;
	/** Left Mouskey State*/
	bool	m_lbuttondown;
	/** Mousewheel State*/
	bool	m_wheel_moved;


	// Only Derived Classes Are Allowed To Instanciate This Class (e.g. class GL_Example)
	/** Class Constructor*/
	GL_Application(const char * class_name);	
	/** Window Class*/
	GL_Window	m_Window;	
	/** Is application initialized?*/
	bool initialized;

	/** CrazyEddies GUI System: OpenGL Renderer */
	CEGUI::OpenGLRenderer *m_GLrenderer;

	/** Mousemove z-delta*/
	short		m_zDelta	;

private:
	/** Program Main Loop*/
	friend int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow);
	/** Program Main Loop*/
	int		Main(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow);
	/** Message Handler*/
	friend LRESULT CALLBACK WindowProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
	/** Message Handler*/
	LRESULT	Message(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);

	/** Application Define Message For Toggling*/
	static const UINT WM_TOGGLEFULLSCREEN = (WM_USER + 1);		

	/** Application ClassName*/
	const char * m_ClassName;	
	/** Window Creation Loop, For FullScreen / Windowed Toggle*/
	bool		m_IsProgramLooping;							
	/** If true, Then Create Fullscreen*/
	bool		m_CreateFullScreen;							
		/** Application Window Visible*/
	bool		m_IsVisible;								
	/** Call Draw() During Window Resize*/
	bool		m_ResizeDraw;								
	/** Last Tick Count*/
	DWORD		m_LastTickCount;							

	

};


#endif	// __GL_FRAMEWORK_H__
